﻿using PuppeteerSharp;

namespace StationProbe
{

    internal enum ElementType
    {
        Page,
        Frame
    }

    internal static class ElementConverter
    {
        public static T? As<T>(this object obj) where T: class
        {
            return obj as T;
        }
    }

    internal class ElementCollection
    {
        private readonly Dictionary<ElementType, object> _objects = new();

        public object this[ElementType type] => _objects[type];

        public void AddObject(ElementType element, object obj)
        {
            _objects.Add(element, obj);
        }
    }

    internal abstract class ProbeTask
    {
        private static string _debugFolder = string.Empty;
        protected readonly Database _db;

        protected ScreenshotOptions _snapshotOpts = new ScreenshotOptions() {Type = ScreenshotType.Jpeg };
        public static bool DebugMode { get; set; }


        public ProbeTask(Database db)
        {
            _db = db;
        }

        public static void InitDebugFolder()
        {
            var drives = DriveInfo.GetDrives();
            var largestDriver = drives.OrderByDescending(x => x.TotalFreeSpace).First();
            var debugFolder = largestDriver.Name + "debug";
            if (!Directory.Exists(debugFolder))
            {
                Directory.CreateDirectory(debugFolder);
            }
            var i = 1;
            while (Directory.Exists(Path.Combine(debugFolder, i.ToString("D6"))))
            {
                i++;
            }
            _debugFolder = Path.Combine(debugFolder, i.ToString("D6"));
            Directory.CreateDirectory(_debugFolder);
        }

        public void SaveDebugSnapshot(string name, byte[] imageData)
        {
            if (Directory.Exists(_debugFolder))
            {
                var file = Path.Combine(_debugFolder, name);
                File.WriteAllBytes(file, imageData);
            }
        }

        public abstract Task<ElementCollection> ExecuteAsync(ElementCollection inputs);
    }

    internal class LoginTask: ProbeTask
    {
        private readonly string _loginUrl;

        public LoginTask(Database db, string loginUrl) :base(db)
        {
            _loginUrl = loginUrl;
        }

        public override async Task<ElementCollection> ExecuteAsync(ElementCollection inputs)
        {
            var page = inputs[ElementType.Page].As<IPage>();
            if (page != null)
            {
                await page.GoToAsync(_loginUrl);
                await page.WaitForNetworkIdleAsync();

                if(DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "100000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }

                //username
                await page.WaitForSelectorAsync("input[_bl_13]");
                await page.FocusAsync("input[_bl_13]");
                await page.Keyboard.TypeAsync("justin.xing");

                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "200000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }

                //password
                await page.WaitForSelectorAsync("input[_bl_14]");
                await page.FocusAsync("input[_bl_14]");
                await page.Keyboard.TypeAsync("123456");

                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "300000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }

                //login
                var button = await page.WaitForSelectorAsync("button[_bl_15]");
                await button.ClickAsync();
                await page.WaitForNavigationAsync();
                await page.WaitForNetworkIdleAsync();

                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "400000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }
            }
            return inputs;
        }
    }

    internal class MenuTask:ProbeTask
    {
        public MenuTask(Database db) : base(db)
        {
        }

        public override async Task<ElementCollection> ExecuteAsync(ElementCollection inputs)
        {
            Logger.WriteLine("Finding menu...");
            var page = inputs[ElementType.Page].As<IPage>();
            if (page != null)
            {
                var frame = await page.WaitForFrameAsync("");
                inputs.AddObject(ElementType.Frame, frame);

                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "500000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }
            }
            return inputs;
        }
    }

    internal class FilterTask : ProbeTask
    {
        private readonly DateTime _start;
        private readonly DateTime _end;

        public int PageCount { get; private set; }

        public int ExamCount { get; private set; }

        public FilterTask(Database db, DateTime start, DateTime end):base(db)
        {
            _start = start;
            _end = end;
        }

        public override async Task<ElementCollection> ExecuteAsync(ElementCollection inputs)
        {
            var page = inputs[ElementType.Page].As<IPage>();
            var frame = inputs[ElementType.Frame].As<IFrame>();
            if(page!= null && frame != null) 
            {
                //show filter dialog
                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "600000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }

                //input start
                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "700000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }

                //input end
                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "800000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }

                await frame.WaitForSelectorAsync("input[_bl_13]");
                await frame.FocusAsync("input[_bl_13]");
                await page.Keyboard.TypeAsync("justin.xing");

                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "900000000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }
            }
            return inputs;
        }
    }

    internal class PageTask:ProbeTask
    {
        private readonly int _page;
        public PageTask(Database db, int page):base(db)
        {
            _page = page;
        }

        public override async Task<ElementCollection> ExecuteAsync(ElementCollection inputs)
        {
            var page = inputs[ElementType.Page].As<IPage>();
            var frame = inputs[ElementType.Frame].As<IFrame>();
            if (page != null && frame != null)
            {
                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "9" + _page.ToString("D5") +"000.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }
            }
            return inputs;
        }
    }

    internal class ExamTask:ProbeTask
    {
        private readonly int _batchTaskId;
        private readonly int _pageIndex;
        private readonly int _examIndex;
        private readonly int _examIndexInPage;

        public ExamTask(Database db, int batchTaskId, int pageIndex, int examIndex, int examIndexInPage): base(db)
        {
            _batchTaskId = batchTaskId;
            _pageIndex = pageIndex;
            _examIndex = examIndex;
            _examIndexInPage = examIndexInPage;
        }

        public override async Task<ElementCollection> ExecuteAsync(ElementCollection inputs)
        {
            var page = inputs[ElementType.Page].As<IPage>();
            var frame = inputs[ElementType.Frame].As<IFrame>();
            if (page != null && frame != null)
            {   //Enter exam

                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "9" + _pageIndex.ToString("D5") + _examIndexInPage.ToString("D3") +".jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }

                var exmaId = Guid.NewGuid().ToString("N");
                
                //Get patient info
                var patientName = "";
                var patientSex = "";
                var patientAge = 0;
                var examDate = DateTime.Now;

                //Get report
                var report = "";

                //Save images.
                var imageUrls = new List<string>();
                for (var i = 0; i < imageUrls.Count; i++)
                {
                    var response = await page.WaitForResponseAsync(imageUrls[i]);
                    var buffer = await response.BufferAsync();
                    var image = new Image()
                    {
                        Id = Guid.NewGuid().ToString("N"),
                        ExamId = exmaId,
                        ImageIndex = i,
                        Data = buffer,
                        CreateTime = DateTime.Now,
                    };
                    _db.AddImage(image);
                }

                //Save exam.
                var exam = new Exam()
                {
                    Id = exmaId,
                    PatientName = patientName,
                    PatientSex = patientSex,
                    PatientAge = patientAge,
                    ExamDate = examDate,
                    Report = report,
                    BatchTaskId = _batchTaskId,
                    ExamIndex = _examIndex,
                    PageIndex = _pageIndex,
                    ExamIndexInPage = _examIndexInPage,
                    CreateTime = DateTime.Now
                };
                _db.AddExam(exam);
                //Back to page.
                await page.GoBackAsync();
                if (DebugMode)
                {
                    var imageData = await page.ScreenshotDataAsync(_snapshotOpts);
                    var fileName = "9" + _pageIndex.ToString("D5") + "999.jpg";
                    SaveDebugSnapshot(fileName, imageData);
                }
            }
            return inputs;
        }
    }
}
